from typing import Iterator

import pytest
import sympy as sp

from src.math_tompy.exceptions import EmptyListError
from src.math_tompy.symbolic_point2d import centroid


def test_centroid_empty_failure():
    # Setup
    points0: list[sp.Point2D] = []

    # Validation
    with pytest.raises(EmptyListError):
        _: sp.Point2D = centroid(points=points0)


def test_centroid_middle_success():
    # Setup
    point0: sp.Point2D = sp.Point2D(1, 1)
    point1: sp.Point2D = sp.Point2D(-1, 1)
    point2: sp.Point2D = sp.Point2D(-1, -1)
    point3: sp.Point2D = sp.Point2D(1, -1)
    point4: sp.Point2D = sp.Point2D(0, 0)
    points0: list[sp.Point2D] = [point0, point1, point2, point3]

    # Execution
    point5: sp.Point2D = centroid(points=points0)

    # Validation
    assert point4 == point5


def test_centroid_iterator_success():
    # Setup
    point0: sp.Point2D = sp.Point2D(1, 1)
    point1: sp.Point2D = sp.Point2D(-1, 1)
    point2: sp.Point2D = sp.Point2D(-1, -1)
    point3: sp.Point2D = sp.Point2D(1, -1)
    point4: sp.Point2D = sp.Point2D(0, 0)
    points0: Iterator[sp.Point2D] = (point for point in [point0, point1, point2, point3])

    # Execution
    point5: sp.Point2D = centroid(points=points0)

    # Validation
    assert point4 == point5


def test_centroid_q1_success():
    # Setup
    point0: sp.Point2D = sp.Point2D(4, 5)
    point1: sp.Point2D = sp.Point2D(7, 5)
    point2: sp.Point2D = sp.Point2D(3, 7)
    point3: sp.Point2D = sp.Point2D(6, 7)
    point4: sp.Point2D = sp.Point2D(5, 6)
    points0: list[sp.Point2D] = [point0, point1, point2, point3]

    # Execution
    point5: sp.Point2D = centroid(points=points0)

    # Validation
    assert point4 == point5


def test_centroid_q3_success():
    # Setup
    point0: sp.Point2D = sp.Point2D(-4, -4)
    point1: sp.Point2D = sp.Point2D(-5, -4)
    point2: sp.Point2D = sp.Point2D(-6, -4)
    point3: sp.Point2D = sp.Point2D(-4, -5)
    point4: sp.Point2D = sp.Point2D(-6, -5)
    point5: sp.Point2D = sp.Point2D(-4, -6)
    point6: sp.Point2D = sp.Point2D(-5, -6)
    point7: sp.Point2D = sp.Point2D(-6, -6)
    point8: sp.Point2D = sp.Point2D(-5, -5)
    points0: list[sp.Point2D] = [point0, point1, point2, point3, point4, point5, point6, point7]

    # Execution
    point9: sp.Point2D = centroid(points=points0)

    # Validation
    assert point8 == point9


def test_centroid_q2_q3_success():
    # Setup
    point0: sp.Point2D = sp.Point2D(-1, 2)
    point1: sp.Point2D = sp.Point2D(-1, -3)
    point2: sp.Point2D = sp.Point2D(-2, 6)
    point3: sp.Point2D = sp.Point2D(-3, 8)
    point4: sp.Point2D = sp.Point2D(-3, -4)
    point5: sp.Point2D = sp.Point2D(-4, -2)
    point6: sp.Point2D = sp.Point2D(-5, 2)
    point7: sp.Point2D = sp.Point2D(-5, 7)
    point8: sp.Point2D = sp.Point2D(-3, 2)
    points0: list[sp.Point2D] = [point0, point1, point2, point3, point4, point5, point6, point7]

    # Execution
    point9: sp.Point2D = centroid(points=points0)

    # Validation
    assert point8 == point9


def test_centroid_q4_q1_success():
    # Setup
    point0: sp.Point2D = sp.Point2D(-4, -4)
    point1: sp.Point2D = sp.Point2D(-5, -4)
    point2: sp.Point2D = sp.Point2D(-6, -4)
    point3: sp.Point2D = sp.Point2D(-4, -5)
    point4: sp.Point2D = sp.Point2D(-6, -5)
    point5: sp.Point2D = sp.Point2D(-4, -6)
    point6: sp.Point2D = sp.Point2D(-5, -6)
    point7: sp.Point2D = sp.Point2D(-6, -6)
    point8: sp.Point2D = sp.Point2D(-5, -5)
    points0: list[sp.Point2D] = [point0, point1, point2, point3, point4, point5, point6, point7]

    # Execution
    point9: sp.Point2D = centroid(points=points0)

    # Validation
    assert point8 == point9



def factorize(number: int) -> list[int]:
    factors: list[int] = [divisor
                          for divisor in range(1, number + 1)
                          if number % divisor == 0]
    return factors



import sympy as sp

from src.math_tompy.symbolic_point2d import shape_node_positions


def test_shape_node_positions_1_success():
    # Setup
    edges0: int = 1
    radius0: sp.S = sp.S(1)
    direction0: sp.Point2D = sp.Point2D(5, 0)
    points0: list[sp.Point2D] = [sp.Point2D(1, 0)]

    # Execution
    points1: list[sp.Point2D] = shape_node_positions(edges=edges0, radius=radius0, direction=direction0)

    # Validation
    assert points0 == points1


def test_shape_node_positions_2_success():
    # Setup
    edges0: int = 2
    radius0: sp.S = sp.S(2)
    direction0: sp.Point2D = sp.Point2D(0, -5)
    points0: list[sp.Point2D] = [sp.Point2D(0, -2), sp.Point2D(0, 2)]

    # Execution
    points1: list[sp.Point2D] = shape_node_positions(edges=edges0, radius=radius0, direction=direction0)

    # Validation
    assert points0 == points1


def test_shape_node_positions_3_success():
    # Setup
    edges0: int = 3
    radius0: sp.S = sp.S(1)
    direction0: sp.Point2D = sp.Point2D(-24, 0)
    points0: list[sp.Point2D] = [sp.Point2D(-1, 0), sp.Point2D(0.5, -sp.sqrt(3)/2), sp.Point2D(0.5, sp.sqrt(3)/2)]

    # Execution
    points1: list[sp.Point2D] = shape_node_positions(edges=edges0, radius=radius0, direction=direction0)

    # Validation
    assert points0 == points1


def test_shape_node_positions_4_success():
    # Setup
    edges0: int = 4
    radius0: float = 2 * sp.sqrt(2)
    direction0: sp.Point2D = sp.Point2D(3, 3)
    points0: list[sp.Point2D] = [sp.Point2D(2, 2),
                                 sp.Point2D(-2, 2),
                                 sp.Point2D(-2, -2),
                                 sp.Point2D(2, -2)]

    # Execution
    points1: list[sp.Point2D] = shape_node_positions(edges=edges0, radius=radius0, direction=direction0)

    # Validation
    assert points0 == points1



import math
from decimal import Decimal

import pytest

from src.math_tompy.decimal import asin


def test_asin_ratio_one_success():
    # Setup
    ratio0: Decimal = Decimal("1")
    angle0: Decimal = Decimal(90 / 180 * math.pi)

    # Execution
    angle1: Decimal = asin(ratio=ratio0)

    # Validation
    assert abs(angle0 - angle1) < Decimal("0.000000000000001")


def test_asin_ratio_sqrt_two_halves_success():
    # Setup
    ratio0: Decimal = Decimal(math.sqrt(2) / 2)
    angle0: Decimal = Decimal(45 / 180 * math.pi)

    # Execution
    angle1: Decimal = asin(ratio=ratio0)

    # Validation
    assert abs(angle0 - angle1) < Decimal("0.000000000000001")


def test_asin_ratio_sqrt_three_halves_success():
    # Setup
    ratio0: Decimal = Decimal(math.sqrt(3) / 2)
    angle0: Decimal = Decimal(60 / 180 * math.pi)

    # Execution
    angle1: Decimal = asin(ratio=ratio0)

    # Validation
    assert abs(angle0 - angle1) < Decimal("0.000000000000001")


def test_asin_ratio_sqrt_four_fourths_success():
    # Setup
    ratio0: Decimal = Decimal(math.sqrt(4) / 4)
    angle0: Decimal = Decimal(30 / 180 * math.pi)

    # Execution
    angle1: Decimal = asin(ratio=ratio0)

    # Validation
    assert abs(angle0 - angle1) < Decimal("0.000000000000001")


# def test_asin_ratio_near_one_failure():
#     # Setup
#     ratio0: Decimal = Decimal("0.975")
#
#     # Validation
#     with pytest.raises(ValueError):
#         _: Decimal = asin(ratio=ratio0)
